@@ -4,6 +4,15 @@ if [ -s "$REDHAT/linux-kernel-test.patch" ]; then
 	exit 1;
 fi
 
+# If DERIVATIVE_STREAM is enabled, increment DERIVATIVE_BUILD and exit
+# (skip RHEL_RELEASE increment for derivative streams).
+if [ "${DERIVATIVE_STREAM}" == "yes" ]; then
+	DERIVATIVE_BUILD="$((DERIVATIVE_BUILD + 1))";
+	sed -i -e "s/^DERIVATIVE_BUILD\ =.*/DERIVATIVE_BUILD = $DERIVATIVE_BUILD/" "$REDHAT"/../Makefile.rhelver;
+	echo "DERIVATIVE_BUILD set to ${DERIVATIVE_BUILD}"
+	exit 0
+fi
+
 RELEASE=$(sed -n -e 's/^RHEL_RELEASE\ =\ \(.*\)/\1/p' "$REDHAT"/../Makefile.rhelver)
 
 YVER=$(echo "$RELEASE" | cut -d "." -f 1)
@@ -13,9 +22,7 @@ ZMAJ=${ZMAJ:=0}
 ZMIN=$(echo "$RELEASE" | cut -s -d "." -f 3)
 ZMIN=${ZMIN:=0}
 
-if [ "$BUMP_RELEASE" == "no" ]; then
-	NEW_RELEASE="$RELEASE";
-elif [ "$ZSTREAM_FLAG" == "no" ]; then
+if [ "$ZSTREAM_FLAG" == "no" ]; then
 	if [ "$YSTREAM_FLAG" == "yes" ]; then
 		NEW_RELEASE="$((RELEASE + 1))";
 	fi
